rm(list=ls())

library(zoo)
library(tibble)
library(Metrics)
library(verification)
library(fields)
library(reshape2)
library(base)
library(plgp)
library(ecodist)
library(mvtnorm)
library(lhs)
library(mda)
library(laGP)
library(zoo)
library(Matrix)
library(optimbase)
eps <- sqrt(.Machine$double.eps) ## defining a small number

load("C:/Users/mapso/OneDrive/Desktop/bias correction/Final_code_data/before_analysis_data_load.RData")
# load the data
# measured temperature in 2011, 2013, and 2017: EN.11, EN.13, EN.17
# WRF/UCM predicted temperature: WRF.11, WRF.13, WRF.17
# Bias (EN - WRF/UCM): Bias.11, Bias.13, Bias.17
# location information (longitude and latitude): geo.11, geo.13, geo.17
# the number of locations in each year: S.11, S.13, S.17 

source("C:/Users/mapso/OneDrive/Desktop/bias correction/Final_code_data/util/nl.R")
source("C:/Users/mapso/OneDrive/Desktop/bias correction/Final_code_data/util/na_extrapolate.R")

################################
# remove missing / outlier data
################################
WRF.11 = WRF.11[,-c(9,14)] #missing values: RNDFB / outlier stations: LANDR
EN.11 = EN.11[,-c(9,14)]
Bias.11 = Bias.11[,-c(9,14)]
location.11 = location.11[-c(9,14),]
geo.11 = geo.11[-c(9,14),]

S.11 = nrow(location.11)
rownames(location.11) = colnames(EN.11)[1:S.11]
S.11

#############################################################################################################
# GP: product 
#############################################################################################################

# load the predictive mean and variance of the tempearautre x*
WRF.pred.11 = read.csv("C:/Users/mapso/OneDrive/Desktop/bias correction/Final_code_data/WRF_pred_11_GP_m.csv")
WRF.pred.13 = read.csv("C:/Users/mapso/OneDrive/Desktop/bias correction/Final_code_data/WRF_pred_13_GP_m.csv")
WRF.pred.17 = read.csv("C:/Users/mapso/OneDrive/Desktop/bias correction/Final_code_data/WRF_pred_17_GP_m.csv")

WRF.pred.var.11 = read.csv("C:/Users/mapso/OneDrive/Desktop/bias correction/Final_code_data/WRF_pred_var_11_GP_m.csv")
WRF.pred.var.13 = read.csv("C:/Users/mapso/OneDrive/Desktop/bias correction/Final_code_data/WRF_pred_var_13_GP_m.csv")
WRF.pred.var.17 = read.csv("C:/Users/mapso/OneDrive/Desktop/bias correction/Final_code_data/WRF_pred_var_17_GP_m.csv")

WRF.pred.11 = WRF.pred.11[,-1]
WRF.pred.13 = WRF.pred.13[,-1]
WRF.pred.17 = WRF.pred.17[,-1]

WRF.pred.var.11 = WRF.pred.var.11[,-1]
WRF.pred.var.13 = WRF.pred.var.13[,-1]
WRF.pred.var.17 = WRF.pred.var.17[,-1]

colnames(WRF.pred.11) = colnames(WRF.11)
colnames(WRF.pred.13) = colnames(WRF.13)
colnames(WRF.pred.17) = colnames(WRF.17)

colnames(WRF.pred.var.11) = colnames(WRF.11)
colnames(WRF.pred.var.13) = colnames(WRF.13)
colnames(WRF.pred.var.17) = colnames(WRF.17)

WRF.pred.11 = as.matrix(WRF.pred.11)
WRF.pred.13 = as.matrix(WRF.pred.13)
WRF.pred.17 = as.matrix(WRF.pred.17)

WRF.pred.var.11 = as.matrix(WRF.pred.var.11)
WRF.pred.var.13 = as.matrix(WRF.pred.var.13)
WRF.pred.var.17 = as.matrix(WRF.pred.var.17)

#############################################################
# 2011
#############################################################
y.pred.11 = y.pred.var.11 =  matrix(NA, 24, S.11)
y.pred.wrf.11 = y.pred.wrf.var.11=  matrix(NA, 24, S.11)
ratio.st.11 = matrix(NA, S.11, 3)
ratio.b.11 = matrix(NA, S.11, 3)
hyper.11 = matrix(NA, 8, S.11)
beta.11 = matrix(NA, 2, S.11)

mean.fixed.11 = matrix(NA, 24, S.11)
mean.Cn.0.11 = matrix(NA, 24, S.11)
mean.Cn.1.11 = matrix(NA, 24, S.11)

mean.fixed.wrf.11 = matrix(NA, 24, S.11)
mean.Cn.0.wrf.11 = matrix(NA, 24, S.11)
mean.Cn.1.wrf.11 = matrix(NA, 24, S.11)

for(i in 1:S.11){
  print(i)
  
  y = c(EN.11[,-i])
  x = c(WRF.11[,-i])
  n = length(y)
  x1 = as.matrix(cbind(rep(1,n), na.approx(x)))
  x1.te = as.matrix(cbind(rep(1,24), na.approx(WRF.11[,i])))
  x1.wrf.te = as.matrix(cbind(rep(1,24), na.approx(WRF.pred.11[,i])))
  
  d = location.11[-i,]
  ds = distance(d)
  dt = sqrt(distance(1:24))
  Dt = kronecker(ones(ncol(ds),ncol(ds)), dt)
  
  # prediction
  d.te = location.11[i,]
  Ds.all = kronecker( distance(rbind(d,d.te)), ones(24, 24) )
  Ds.tr = Ds.all[1:((S.11-1)*24), 1:((S.11-1)*24)]
  Ds.trte = Ds.all[((S.11-1)*24+1):(S.11*24), 1:((S.11-1)*24)]
  Ds.tete = Ds.all[((S.11-1)*24+1):(S.11*24), ((S.11-1)*24+1):(S.11*24)]
  
  dt = sqrt(distance(1:24))
  Dt.all = kronecker(ones(S.11,S.11), dt)
  Dt.tr = Dt.all[1:((S.11-1)*24), 1:((S.11-1)*24)]
  Dt.trte = Dt.all[((S.11-1)*24+1):(S.11*24), 1:((S.11-1)*24)]
  Dt.tete = Dt.all[((S.11-1)*24+1):(S.11*24), ((S.11-1)*24+1):(S.11*24)]
  
  counter <- 0
  # out <- optim(rep(0.8,8),
  out <- optim(c(0.1, 0.1, 0.8, 0.8, 0.1*var(y,na.rm=T), 0.1*var(y,na.rm=T), 0.1*var(y,na.rm=T), 0.1*var(y,na.rm=T)),
               nl,
               method="L-BFGS-B",
               lower=c(eps, eps, eps, eps, eps, eps, eps, eps),
               # upper=c(20,20, (1-eps),(1-eps), var(y,na.rm=T),var(y,na.rm=T),var(y,na.rm=T),var(y,na.rm=T)),
               upper=c(Inf,Inf,(1-eps),(1-eps),Inf,Inf,Inf,Inf),
               Ds=Ds.tr,Dt=Dt.tr,Y=y,X=x)
  hyper.11[,i] = out$par
  length(out$par)
  out$par
  theta.s0 = out$par[1]
  theta.s1 = out$par[2]
  rho.0 = out$par[3]
  rho.1 = out$par[4]
  delta.0_tau2 = out$par[5]
  delta.1_tau2 = out$par[6]
  tau2.1_tau2 = out$par[7]
  g = out$par[8]
  
  ################################################################
  # Define the kernel or covariance function for training data
  C0 = exp(-Ds.tr/theta.s0) + delta.0_tau2*(rho.0^Dt.tr)/(1-rho.0^2)
  C1 = tau2.1_tau2 * exp(-Ds.tr/theta.s1) + delta.1_tau2*(rho.1^Dt.tr)/(1-rho.1^2)
  C2 =  diag(g, n)
  K <- C0 + ( diag(na.approx(x)) %*% C1 %*% diag(na.approx(x)) ) + C2 
  #####################################################################################################################
  # trte
  C0.trte = exp(-Ds.trte/theta.s0) + delta.0_tau2*(rho.0^Dt.trte)/(1-rho.0^2)
  C1.trte = tau2.1_tau2 * exp(-Ds.trte/theta.s1) + delta.1_tau2*(rho.1^Dt.trte)/(1-rho.1^2)
  C.trte <- C0.trte + ( diag(na.approx(x1.te[,2])) %*% C1.trte %*% diag(na.approx(x1[,2])) ) 
  # te
  C0.tete = exp(-Ds.tete/theta.s0) + delta.0_tau2*(rho.0^Dt.tete)/(1-rho.0^2)
  C1.tete = tau2.1_tau2 * exp(-Ds.tete/theta.s1) + delta.1_tau2*(rho.1^Dt.tete)/(1-rho.1^2)
  C.tete <- C0.tete + ( diag(na.approx(x1.te[,2])) %*% C1.tete %*% diag(na.approx(x1.te[,2])) )
  
  ###additional covariance term
  C.trte.wrf <- C0.trte + ( diag(WRF.pred.11[,i]) %*% C1.trte %*% diag(na.approx(x1[,2])) ) 
  C.tete.wrf <- C0.tete + ( diag(WRF.pred.11[,i]) %*% C1.tete %*% diag(WRF.pred.11[,i]) )
  ################################################################
  Ki = solve(K)
  
  # Estimation of fixed effect (Mean of Y)
  beta.hat = solve( t(na.approx(x1))%*% Ki%*% na.approx(x1) ) %*% t( na.approx(x1) ) %*% Ki %*% na.approx(y)
  beta.hat
  beta.11[,i] = beta.hat
  
  tau2 = 1/n * t(na.approx(y)-x1%*% beta.hat) %*% Ki %*% (na.approx(y)-x1%*% beta.hat)
  tau2 = as.numeric(tau2)
  Sigmai= Ki/tau2
  
  tau2.1 =  tau2.1_tau2 * tau2
  delta.0 = delta.0_tau2 * tau2
  delta.1 = delta.1_tau2 * tau2
  sigma2 = tau2 * g
  
  #####################################################################################################################
  # Mean of GP
  mean.Cn = C.trte %*% Ki %*% (na.approx(y)-x1 %*% beta.hat)
  #####################################################################################################################
  # original data
  y.hat = x1.te %*% beta.hat + mean.Cn
  y.hat.var = tau2 * as.numeric(diag( C.tete - C.trte %*% Ki %*%  t(C.trte) )) + sigma2
  #############################################
  mean.fixed.11[,i] = x1.te %*% beta.hat
  mean.Cn.0.11[,i] = C0.trte %*% Ki %*% (na.approx(y)-x1 %*% beta.hat)
  mean.Cn.1.11[,i] = ( diag(na.approx(x1.te[,2])) %*% C1.trte %*% diag(na.approx(x1[,2])) ) %*% Ki %*% (na.approx(y)-x1 %*% beta.hat)
  
  #####################################################################################################################
  # Mean of GP
  mean.Cn.wrf = C.trte.wrf %*% Ki %*% (na.approx(y)-x1 %*% beta.hat)
  ########## by the total law of variance
  Ecc.a = tau2*diag(C0.tete) + (WRF.pred.11[,i]^2 + WRF.pred.var.11[,i]) * tau2*diag(C1.tete) + rep(sigma2, ncol(Dt.tete))
  C0starDxC1starmu = t(tau2*C0.trte) + diag(na.approx(x1[,2]))%*%t(tau2*C1.trte)%*% diag(WRF.pred.11[,i])  
  EcSigmac.a = diag(t(C0starDxC1starmu) %*% Sigmai %*% C0starDxC1starmu)
  v.a = WRF.pred.var.11[,i] *(beta.hat[2] + tau2*C1.trte %*% diag(na.approx(x1[,2])) %*% Sigmai %*%  (na.approx(y)-x1%*% beta.hat))^2
  # WRF prediction data
  y.hat.wrf = x1.wrf.te %*% beta.hat + mean.Cn.wrf
  y.hat.wrf.var = Ecc.a - EcSigmac.a + v.a
  #############################################
  mean.fixed.wrf.11[,i] = x1.wrf.te %*% beta.hat
  mean.Cn.0.wrf.11[,i] = C0.trte %*% Ki %*% (na.approx(y)-x1 %*% beta.hat)
  mean.Cn.1.wrf.11[,i] = ( diag(WRF.pred.11[,i]) %*% C1.trte %*% diag(na.approx(x1[,2])) ) %*% Ki %*% (na.approx(y)-x1 %*% beta.hat)
  
  y.pred.11[,i] = y.hat
  y.pred.var.11[,i] = y.hat.var
  y.pred.wrf.11[,i] = y.hat.wrf
  y.pred.wrf.var.11[,i] = y.hat.wrf.var
  
  # y.pred.var.11
  var.tr = tau2 *diag(K) #
  
  variation.s = tau2 + x^2 * tau2.1
  variation.t = delta.0/(1-rho.0^2) + x^2 * delta.1/(1-rho.1^2)
  variation.sigma2 = sigma2
  
  ratio.s = round(mean(variation.s / var.tr * 100, na.rm=T),2)
  ratio.t = round(mean(variation.t / var.tr * 100, na.rm=T),2)
  ratio.sigma2 = round(mean(variation.sigma2 / var.tr * 100, na.rm=T),2)
  
  ratio.st.11[i,] = c(ratio.s, ratio.t, ratio.sigma2)
  
  variation.b0 = tau2 + delta.0/(1-rho.0^2)
  variation.b1 = x^2 * tau2.1 + x^2 * delta.1/(1-rho.1^2)
  
  ratio.b0 =  round(mean(variation.b0 / var.tr * 100, na.rm=T),2)
  ratio.b1 =  round(mean(variation.b1 / var.tr * 100, na.rm=T),2)
  ratio.b.sigma2 = round(mean(variation.sigma2 / var.tr * 100, na.rm=T),2)
  
  ratio.b.11[i,] = c(ratio.b0, ratio.b1, ratio.b.sigma2)
}

round(colMeans(ratio.b.11),2)
round(apply(ratio.b.11, 2,sd),2) 

round(colMeans(ratio.st.11),2)
round(apply(ratio.st.11, 2,sd),2) 

#############################################################
# 2013
#############################################################
y.pred.13 = y.pred.var.13 =  matrix(NA, 24, S.13)
y.pred.wrf.13 = y.pred.wrf.var.13=  matrix(NA, 24, S.13)
ratio.st.13 = matrix(NA, S.13, 3)
ratio.b.13 = matrix(NA, S.13, 3)
hyper.13 = matrix(NA, 8, S.13)
beta.13 = matrix(NA, 2, S.13)

mean.fixed.13 = matrix(NA, 24, S.13)
mean.Cn.0.13 = matrix(NA, 24, S.13)
mean.Cn.1.13 = matrix(NA, 24, S.13)

mean.fixed.wrf.13 = matrix(NA, 24, S.13)
mean.Cn.0.wrf.13 = matrix(NA, 24, S.13)
mean.Cn.1.wrf.13 = matrix(NA, 24, S.13)

for(i in 1:S.13){
  print(i)
  
  y = c(EN.13[,-i])
  x = c(WRF.13[,-i])
  n = length(y)
  x1 = as.matrix(cbind(rep(1,n), na.approx(x)))
  x1.te = as.matrix(cbind(rep(1,24), na.approx(WRF.13[,i])))
  x1.wrf.te = as.matrix(cbind(rep(1,24), na.approx(WRF.pred.13[,i])))
  
  d = location.13[-i,]
  ds = distance(d)
  dt = sqrt(distance(1:24))
  Dt = kronecker(ones(ncol(ds),ncol(ds)), dt)
  
  # prediction
  d.te = location.13[i,]
  Ds.all = kronecker( distance(rbind(d,d.te)), ones(24, 24) )
  Ds.tr = Ds.all[1:((S.13-1)*24), 1:((S.13-1)*24)]
  Ds.trte = Ds.all[((S.13-1)*24+1):(S.13*24), 1:((S.13-1)*24)]
  Ds.tete = Ds.all[((S.13-1)*24+1):(S.13*24), ((S.13-1)*24+1):(S.13*24)]
  
  dt = sqrt(distance(1:24))
  Dt.all = kronecker(ones(S.13,S.13), dt)
  Dt.tr = Dt.all[1:((S.13-1)*24), 1:((S.13-1)*24)]
  Dt.trte = Dt.all[((S.13-1)*24+1):(S.13*24), 1:((S.13-1)*24)]
  Dt.tete = Dt.all[((S.13-1)*24+1):(S.13*24), ((S.13-1)*24+1):(S.13*24)]
  
  # c(0.1, 0.1, 0.9, 0.9, 0.1*var(y,na.rm=T), 0.1*var(y,na.rm=T), 0.1*var(y,na.rm=T), 0.1*var(y,na.rm=T))
  counter <- 0
  # out <- optim(rep(0.8,8),
  out <- optim(c(0.1, 0.1, 0.8, 0.8, 0.1*var(y,na.rm=T), 0.1*var(y,na.rm=T), 0.1*var(y,na.rm=T), 0.1*var(y,na.rm=T)),
               nl,
               method="L-BFGS-B",
               lower=c(eps, eps, eps, eps, eps, eps, eps, eps),
               # upper=c(20,20, (1-eps),(1-eps), var(y,na.rm=T),var(y,na.rm=T),var(y,na.rm=T),var(y,na.rm=T)),
               upper=c(Inf,Inf,(1-eps),(1-eps),Inf,Inf,Inf,Inf),
               Ds=Ds.tr,Dt=Dt.tr,Y=y,X=x)
  hyper.13[,i] = out$par
  length(out$par)
  out$par
  theta.s0 = out$par[1]
  theta.s1 = out$par[2]
  rho.0 = out$par[3]
  rho.1 = out$par[4]
  delta.0_tau2 = out$par[5]
  delta.1_tau2 = out$par[6]
  tau2.1_tau2 = out$par[7]
  g = out$par[8]
  
  ################################################################
  ################################################################
  # Define the kernel or covariance function for training data
  C0 = exp(-Ds.tr/theta.s0) + delta.0_tau2*(rho.0^Dt.tr)/(1-rho.0^2)
  C1 = tau2.1_tau2 * exp(-Ds.tr/theta.s1) + delta.1_tau2*(rho.1^Dt.tr)/(1-rho.1^2)
  C2 =  diag(g, n)
  K <- C0 + ( diag(na.approx(x)) %*% C1 %*% diag(na.approx(x)) ) + C2 
  #####################################################################################################################
  # trte
  C0.trte = exp(-Ds.trte/theta.s0) + delta.0_tau2*(rho.0^Dt.trte)/(1-rho.0^2)
  C1.trte = tau2.1_tau2 * exp(-Ds.trte/theta.s1) + delta.1_tau2*(rho.1^Dt.trte)/(1-rho.1^2)
  C.trte <- C0.trte + ( diag(na.approx(x1.te[,2])) %*% C1.trte %*% diag(na.approx(x1[,2])) ) 
  # te
  C0.tete = exp(-Ds.tete/theta.s0) + delta.0_tau2*(rho.0^Dt.tete)/(1-rho.0^2)
  C1.tete = tau2.1_tau2 * exp(-Ds.tete/theta.s1) + delta.1_tau2*(rho.1^Dt.tete)/(1-rho.1^2)
  C.tete <- C0.tete + ( diag(na.approx(x1.te[,2])) %*% C1.tete %*% diag(na.approx(x1.te[,2])) )
  
  ###additional covariance term
  C.trte.wrf <- C0.trte + ( diag(WRF.pred.13[,i]) %*% C1.trte %*% diag(na.approx(x1[,2])) ) 
  C.tete.wrf <- C0.tete + ( diag(WRF.pred.13[,i]) %*% C1.tete %*% diag(WRF.pred.13[,i]) )
  ################################################################
  Ki = solve(K)
  
  # Estimation of fixed effect (Mean of Y)
  beta.hat = solve( t(na.approx(x1))%*% Ki%*% na.approx(x1) ) %*% t( na.approx(x1) ) %*% Ki %*% na.approx(y)
  beta.hat
  beta.13[,i] = beta.hat
  
  tau2 = 1/n * t(na.approx(y)-x1%*% beta.hat) %*% Ki %*% (na.approx(y)-x1%*% beta.hat)
  tau2 = as.numeric(tau2)
  Sigmai= Ki/tau2
  
  tau2.1 =  tau2.1_tau2 * tau2
  delta.0 = delta.0_tau2 * tau2
  delta.1 = delta.1_tau2 * tau2
  sigma2 = tau2 * g
  
  #####################################################################################################################
  # Mean of GP
  mean.Cn = C.trte %*% Ki %*% (na.approx(y)-x1 %*% beta.hat)
  #####################################################################################################################
  # original data
  y.hat = x1.te %*% beta.hat + mean.Cn
  y.hat.var = tau2 * as.numeric(diag( C.tete - C.trte %*% Ki %*%  t(C.trte) )) + sigma2
  #############################################
  mean.fixed.13[,i] = x1.te %*% beta.hat
  mean.Cn.0.13[,i] = C0.trte %*% Ki %*% (na.approx(y)-x1 %*% beta.hat)
  mean.Cn.1.13[,i] = ( diag(na.approx(x1.te[,2])) %*% C1.trte %*% diag(na.approx(x1[,2])) ) %*% Ki %*% (na.approx(y)-x1 %*% beta.hat)
  
  #####################################################################################################################
  # Mean of GP
  mean.Cn.wrf = C.trte.wrf %*% Ki %*% (na.approx(y)-x1 %*% beta.hat)
  ########## by the total law of variance
  Ecc.a = tau2*diag(C0.tete) + (WRF.pred.13[,i]^2 + WRF.pred.var.13[,i]) * tau2*diag(C1.tete) + rep(sigma2, ncol(Dt.tete))
  C0starDxC1starmu = t(tau2*C0.trte) + diag(na.approx(x1[,2]))%*%t(tau2*C1.trte)%*% diag(WRF.pred.13[,i])  
  EcSigmac.a = diag(t(C0starDxC1starmu) %*% Sigmai %*% C0starDxC1starmu)
  v.a = WRF.pred.var.13[,i] *(beta.hat[2] + tau2*C1.trte %*% diag(na.approx(x1[,2])) %*% Sigmai %*%  (na.approx(y)-x1%*% beta.hat))^2
  # WRF prediction data
  y.hat.wrf = x1.wrf.te %*% beta.hat + mean.Cn.wrf
  y.hat.wrf.var = Ecc.a - EcSigmac.a + v.a
  #############################################
  mean.fixed.wrf.13[,i] = x1.wrf.te %*% beta.hat
  mean.Cn.0.wrf.13[,i] = C0.trte %*% Ki %*% (na.approx(y)-x1 %*% beta.hat)
  mean.Cn.1.wrf.13[,i] = ( diag(WRF.pred.13[,i]) %*% C1.trte %*% diag(na.approx(x1[,2])) ) %*% Ki %*% (na.approx(y)-x1 %*% beta.hat)
  
  y.pred.13[,i] = y.hat
  y.pred.var.13[,i] = y.hat.var
  y.pred.wrf.13[,i] = y.hat.wrf
  y.pred.wrf.var.13[,i] = y.hat.wrf.var
  
  # y.pred.var.13
  var.tr = tau2 *diag(K) #
  
  variation.s = tau2 + x^2 * tau2.1
  variation.t = delta.0/(1-rho.0^2) + x^2 * delta.1/(1-rho.1^2)
  variation.sigma2 = sigma2
  
  ratio.s = round(mean(variation.s / var.tr * 100, na.rm=T),2)
  ratio.t = round(mean(variation.t / var.tr * 100, na.rm=T),2)
  ratio.sigma2 = round(mean(variation.sigma2 / var.tr * 100, na.rm=T),2)
  
  ratio.st.13[i,] = c(ratio.s, ratio.t, ratio.sigma2)
  
  variation.b0 = tau2 + delta.0/(1-rho.0^2)
  variation.b1 = x^2 * tau2.1 + x^2 * delta.1/(1-rho.1^2)
  
  ratio.b0 =  round(mean(variation.b0 / var.tr * 100, na.rm=T),2)
  ratio.b1 =  round(mean(variation.b1 / var.tr * 100, na.rm=T),2)
  ratio.b.sigma2 = round(mean(variation.sigma2 / var.tr * 100, na.rm=T),2)
  
  ratio.b.13[i,] = c(ratio.b0, ratio.b1, ratio.b.sigma2)
}

#############################################################
# 2017
#############################################################
y.pred.17 = y.pred.var.17 =  matrix(NA, 24, S.17)
y.pred.wrf.17 = y.pred.wrf.var.17=  matrix(NA, 24, S.17)
ratio.st.17 = matrix(NA, S.17, 3)
ratio.b.17 = matrix(NA, S.17, 3)
hyper.17 = matrix(NA, 8, S.17)
beta.17 = matrix(NA, 2, S.17)

mean.fixed.17 = matrix(NA, 24, S.17)
mean.Cn.0.17 = matrix(NA, 24, S.17)
mean.Cn.1.17 = matrix(NA, 24, S.17)

mean.fixed.wrf.17 = matrix(NA, 24, S.17)
mean.Cn.0.wrf.17 = matrix(NA, 24, S.17)
mean.Cn.1.wrf.17 = matrix(NA, 24, S.17)

for(i in 1:S.17){
  print(i)
  
  y = c(EN.17[,-i])
  x = c(WRF.17[,-i])
  n = length(y)
  x1 = as.matrix(cbind(rep(1,n), na.approx(x)))
  x1.te = as.matrix(cbind(rep(1,24), na.approx(WRF.17[,i])))
  x1.wrf.te = as.matrix(cbind(rep(1,24), na.approx(WRF.pred.17[,i])))
  
  d = location.17[-i,]
  ds = distance(d)
  dt = sqrt(distance(1:24))
  Dt = kronecker(ones(ncol(ds),ncol(ds)), dt)
  
  # prediction
  d.te = location.17[i,]
  Ds.all = kronecker( distance(rbind(d,d.te)), ones(24, 24) )
  Ds.tr = Ds.all[1:((S.17-1)*24), 1:((S.17-1)*24)]
  Ds.trte = Ds.all[((S.17-1)*24+1):(S.17*24), 1:((S.17-1)*24)]
  Ds.tete = Ds.all[((S.17-1)*24+1):(S.17*24), ((S.17-1)*24+1):(S.17*24)]
  
  dt = sqrt(distance(1:24))
  Dt.all = kronecker(ones(S.17,S.17), dt)
  Dt.tr = Dt.all[1:((S.17-1)*24), 1:((S.17-1)*24)]
  Dt.trte = Dt.all[((S.17-1)*24+1):(S.17*24), 1:((S.17-1)*24)]
  Dt.tete = Dt.all[((S.17-1)*24+1):(S.17*24), ((S.17-1)*24+1):(S.17*24)]
  
  # c(0.1, 0.1, 0.9, 0.9, 0.1*var(y,na.rm=T), 0.1*var(y,na.rm=T), 0.1*var(y,na.rm=T), 0.1*var(y,na.rm=T))
  counter <- 0
  # out <- optim(rep(0.8,8),
  out <- optim(c(0.1, 0.1, 0.8, 0.8, 0.1*var(y,na.rm=T), 0.1*var(y,na.rm=T), 0.1*var(y,na.rm=T), 0.1*var(y,na.rm=T)),
               nl,
               method="L-BFGS-B",
               lower=c(eps, eps, eps, eps, eps, eps, eps, eps),
               # upper=c(20,20, (1-eps),(1-eps), var(y,na.rm=T),var(y,na.rm=T),var(y,na.rm=T),var(y,na.rm=T)),
               upper=c(Inf,Inf,(1-eps),(1-eps),Inf,Inf,Inf,Inf),
               Ds=Ds.tr,Dt=Dt.tr,Y=y,X=x)
  hyper.17[,i] = out$par
  length(out$par)
  out$par
  theta.s0 = out$par[1]
  theta.s1 = out$par[2]
  rho.0 = out$par[3]
  rho.1 = out$par[4]
  delta.0_tau2 = out$par[5]
  delta.1_tau2 = out$par[6]
  tau2.1_tau2 = out$par[7]
  g = out$par[8]
  
  ################################################################
  ################################################################
  # Define the kernel or covariance function for training data
  C0 = exp(-Ds.tr/theta.s0) + delta.0_tau2*(rho.0^Dt.tr)/(1-rho.0^2)
  C1 = tau2.1_tau2 * exp(-Ds.tr/theta.s1) + delta.1_tau2*(rho.1^Dt.tr)/(1-rho.1^2)
  C2 =  diag(g, n)
  K <- C0 + ( diag(na.approx(x)) %*% C1 %*% diag(na.approx(x)) ) + C2 
  #####################################################################################################################
  # trte
  C0.trte = exp(-Ds.trte/theta.s0) + delta.0_tau2*(rho.0^Dt.trte)/(1-rho.0^2)
  C1.trte = tau2.1_tau2 * exp(-Ds.trte/theta.s1) + delta.1_tau2*(rho.1^Dt.trte)/(1-rho.1^2)
  C.trte <- C0.trte + ( diag(na.approx(x1.te[,2])) %*% C1.trte %*% diag(na.approx(x1[,2])) ) 
  # te
  C0.tete = exp(-Ds.tete/theta.s0) + delta.0_tau2*(rho.0^Dt.tete)/(1-rho.0^2)
  C1.tete = tau2.1_tau2 * exp(-Ds.tete/theta.s1) + delta.1_tau2*(rho.1^Dt.tete)/(1-rho.1^2)
  C.tete <- C0.tete + ( diag(na.approx(x1.te[,2])) %*% C1.tete %*% diag(na.approx(x1.te[,2])) )
  
  ###additional covariance term
  C.trte.wrf <- C0.trte + ( diag(WRF.pred.17[,i]) %*% C1.trte %*% diag(na.approx(x1[,2])) ) 
  C.tete.wrf <- C0.tete + ( diag(WRF.pred.17[,i]) %*% C1.tete %*% diag(WRF.pred.17[,i]) )
  ################################################################
  Ki = solve(K)
  
  # Estimation of fixed effect (Mean of Y)
  beta.hat = solve( t(na.approx(x1))%*% Ki%*% na.approx(x1) ) %*% t( na.approx(x1) ) %*% Ki %*% na.approx(y)
  beta.hat
  beta.17[,i] = beta.hat
  
  tau2 = 1/n * t(na.approx(y)-x1%*% beta.hat) %*% Ki %*% (na.approx(y)-x1%*% beta.hat)
  tau2 = as.numeric(tau2)
  Sigmai= Ki/tau2
  
  tau2.1 =  tau2.1_tau2 * tau2
  delta.0 = delta.0_tau2 * tau2
  delta.1 = delta.1_tau2 * tau2
  sigma2 = tau2 * g
  
  #####################################################################################################################
  # Mean of GP
  mean.Cn = C.trte %*% Ki %*% (na.approx(y)-x1 %*% beta.hat)
  #####################################################################################################################
  # original data
  y.hat = x1.te %*% beta.hat + mean.Cn
  y.hat.var = tau2 * as.numeric(diag( C.tete - C.trte %*% Ki %*%  t(C.trte) )) + sigma2
  #############################################
  mean.fixed.17[,i] = x1.te %*% beta.hat
  mean.Cn.0.17[,i] = C0.trte %*% Ki %*% (na.approx(y)-x1 %*% beta.hat)
  mean.Cn.1.17[,i] = ( diag(na.approx(x1.te[,2])) %*% C1.trte %*% diag(na.approx(x1[,2])) ) %*% Ki %*% (na.approx(y)-x1 %*% beta.hat)
  
  #####################################################################################################################
  # Mean of GP
  mean.Cn.wrf = C.trte.wrf %*% Ki %*% (na.approx(y)-x1 %*% beta.hat)
  ########## by the total law of variance
  Ecc.a = tau2*diag(C0.tete) + (WRF.pred.17[,i]^2 + WRF.pred.var.17[,i]) * tau2*diag(C1.tete) + rep(sigma2, ncol(Dt.tete))
  C0starDxC1starmu = t(tau2*C0.trte) + diag(na.approx(x1[,2]))%*%t(tau2*C1.trte)%*% diag(WRF.pred.17[,i])  
  EcSigmac.a = diag(t(C0starDxC1starmu) %*% Sigmai %*% C0starDxC1starmu)
  v.a = WRF.pred.var.17[,i] *(beta.hat[2] + tau2*C1.trte %*% diag(na.approx(x1[,2])) %*% Sigmai %*%  (na.approx(y)-x1%*% beta.hat))^2
  # WRF prediction data
  y.hat.wrf = x1.wrf.te %*% beta.hat + mean.Cn.wrf
  y.hat.wrf.var = Ecc.a - EcSigmac.a + v.a
  #############################################
  mean.fixed.wrf.17[,i] = x1.wrf.te %*% beta.hat
  mean.Cn.0.wrf.17[,i] = C0.trte %*% Ki %*% (na.approx(y)-x1 %*% beta.hat)
  mean.Cn.1.wrf.17[,i] = ( diag(WRF.pred.17[,i]) %*% C1.trte %*% diag(na.approx(x1[,2])) ) %*% Ki %*% (na.approx(y)-x1 %*% beta.hat)
  
  y.pred.17[,i] = y.hat
  y.pred.var.17[,i] = y.hat.var
  y.pred.wrf.17[,i] = y.hat.wrf
  y.pred.wrf.var.17[,i] = y.hat.wrf.var
  
  # y.pred.var.17
  var.tr = tau2 *diag(K) #
  
  variation.s = tau2 + x^2 * tau2.1
  variation.t = delta.0/(1-rho.0^2) + x^2 * delta.1/(1-rho.1^2)
  variation.sigma2 = sigma2
  
  ratio.s = round(mean(variation.s / var.tr * 100, na.rm=T),2)
  ratio.t = round(mean(variation.t / var.tr * 100, na.rm=T),2)
  ratio.sigma2 = round(mean(variation.sigma2 / var.tr * 100, na.rm=T),2)
  
  ratio.st.17[i,] = c(ratio.s, ratio.t, ratio.sigma2)
  
  variation.b0 = tau2 + delta.0/(1-rho.0^2)
  variation.b1 = x^2 * tau2.1 + x^2 * delta.1/(1-rho.1^2)
  
  ratio.b0 =  round(mean(variation.b0 / var.tr * 100, na.rm=T),2)
  ratio.b1 =  round(mean(variation.b1 / var.tr * 100, na.rm=T),2)
  ratio.b.sigma2 = round(mean(variation.sigma2 / var.tr * 100, na.rm=T),2)
  
  ratio.b.17[i,] = c(ratio.b0, ratio.b1, ratio.b.sigma2)
}

# all the results are saved in "results_bias_correct.RData"



